"""Base class for chefs."""

from abc import ABC, abstractmethod
from typing import Any


class BaseChef(ABC):
    """Base class for chefs."""

    @abstractmethod
    def process(self, path: Any) -> Any:
        """Process the data."""
        raise NotImplementedError("Subclasses must implement process()")

    def __call__(self, path: Any) -> Any:
        """Call the chef to process the data."""
        return self.process(path)

    def __repr__(self) -> str:
        """Return a string representation of the chef."""
        return f"{self.__class__.__name__}()"



"""TextChef is a chef that processes text data."""

from pathlib import Path
from typing import List, Union

from .base import BaseChef


class TextChef(BaseChef):
    """TextChef is a chef that processes text data."""

    def process(self, path: Union[str, Path]) -> str:
        """Process the text data from given file(s).

        Args:
            path (Union[str, Path]): Path to the file(s) to process.

        Returns:
            str: Processed text data.

        """
        with open(path, "r") as file:
            return str(file.read())

    def process_batch(self, paths: Union[List[str], List[Path]]) -> List[str]:
        """Process the text data in a batch.

        Args:
            paths (Union[List[str], List[Path]]): Paths to the files to process.

        Returns:
            List[str]: Processed text data.

        """
        return [self.process(path) for path in paths]

    def __call__(
        self, path: Union[str, Path, List[str], List[Path]]
    ) -> Union[str, List[str]]:
        """Process the text data from given file(s)."""
        if isinstance(path, (list, tuple)):
            return self.process_batch(path)
        elif isinstance(path, (str, Path)):
            return self.process(path)
        else:
            raise TypeError(f"Unsupported type: {type(path)}")

    def __repr__(self) -> str:
        """Return a string representation of the chef."""
        return f"{self.__class__.__name__}()"



"""FileFetcher is a fetcher that fetches paths of files from local directories."""

from pathlib import Path
from typing import List, Optional

from .base import BaseFetcher


class FileFetcher(BaseFetcher):
    """FileFetcher is a fetcher that fetches paths of files from local directories."""

    def __init__(self) -> None:
        """Initialize the FileFetcher."""
        super().__init__()

    def fetch(self, dir: str, ext: Optional[List[str]] = None) -> List[Path]:
        """Fetch files from a directory.

        Args:
            dir (str): The directory to fetch files from.
            ext (Optional[List[str]]): The file extensions to fetch.

        Returns:
            List[Path]: The list of files fetched from the directory.

        """
        # Reads the entire directory and returns a list of files with the specified extension
        return [
            file
            for file in Path(dir).iterdir()
            if file.is_file() and (ext is None or file.suffix in ext)
        ]

    def fetch_file(self, dir: str, name: str) -> Path:  # type: ignore[override]
        """Given a directory and a file name, return the path to the file.

        NOTE: This method is mostly for uniformity across fetchers since one may require to
        get a file from an online database.
        """
        # We should search the directory for the file
        for file in Path(dir).iterdir():
            if file.is_file() and file.name == name:
                return file
        raise FileNotFoundError(f"File {name} not found in directory {dir}")

    def __call__(self, dir: str, ext: Optional[List[str]] = None) -> List[Path]:  # type: ignore[override]
        """Fetch files from a directory.

        Args:
            dir (str): The directory to fetch files from.
            ext (Optional[List[str]]): The file extensions to fetch.

        Returns:
            List[Path]: The list of files fetched from the directory.

        """
        return self.fetch(dir, ext)
